/*
	Engineering Test: Boggle Solver
	Gordon Lee

*******************************************************************************

	@filename	cBoard.h

	@description
		Class for a Boggle board of letters

	@author		Gordon Lee
*/

// Includes
#include "cBoard.h"
#include <string>
#include <stdio.h>
#include <ctype.h>

// Namespace
namespace Test
{
////////////////////////////////////////////////////////////////////////////////
/**
	Constructor
*/
cBoard::cBoard()
: m_board(NULL), m_rows(0), m_cols(0)
{
}

/**
	Deconstructor
*/
cBoard::~cBoard()
{
	if (m_board)
		delete [] m_board;
}

/**
	Parse Board Input

	@param char *_input
*/
void cBoard::ParseBoardInput(char *_input)
{
	char *_tokenRow = NULL, *_tokenChar = NULL;
	char *_nextTokenRow = NULL, *_nextTokenChar = NULL;
	size_t _inputLength = strlen(_input);
	int _maxCols = 0;
	int _r = 0, _c = 0;
	size_t _boardLength = 0;

	if (_inputLength == 0)
		throw cBoardException();

	m_board = new char[_inputLength];

	// Split input into row tokens from delimiters of new lines
	_tokenRow = strtok_s(_input, "\r\n", &_nextTokenRow);
	while (_tokenRow)
	{
		int _numColsInRow = 0;
		_c = 0;

		// Split row tokens into char from delimiters of spaces
		_tokenChar = strtok_s(_tokenRow, " ", &_nextTokenChar);
		while (_tokenChar)
		{
			// Invalid char
			if (strlen(_tokenChar) != 1 || tolower(_tokenChar[0]) < 'a' || tolower(_tokenChar[0]) > 'z')
				throw cBoardException();

			m_board[(_r * _maxCols) + _c] = tolower(_tokenChar[0]);
			++_c;
			++_numColsInRow;
			_tokenChar = strtok_s(NULL, " ", &_nextTokenChar);
		}

		// Column checking
		if (_maxCols == 0)
			_maxCols = _numColsInRow;
		else
		{
			if (_numColsInRow > _maxCols || _numColsInRow < _maxCols)
				throw cBoardException();
		}
		
		++_r;
		_tokenRow = strtok_s(NULL, "\r\n", &_nextTokenRow);
	}

	// End
	m_rows = _r;
	m_cols = _maxCols;
	_boardLength = m_rows * m_cols;
	m_board[_boardLength] = '\0';
	m_size = _boardLength;
}

/**
	Rows

	@return int
*/
int cBoard::Rows(void)
{
	return m_rows;
}

/**
	Cols

	@return int
*/
int cBoard::Cols(void)
{
	return m_cols;
}

/**
	Get Char

	@param int _r
	@param int _c

	@return char
*/
char cBoard::GetChar(int _r, int _c)
{
	if (!m_board)
		return NULL;

	return m_board[(_r * m_cols) + _c];
}

/**
	Size

	@return size_t
*/
size_t cBoard::Size(void)
{
	return m_size;
}
////////////////////////////////////////////////////////////////////////////////
}
