/*
	Engineering Test: Boggle Solver
	Gordon Lee

*******************************************************************************

	@filename	cDictionary.cpp

	@description
		Class for a word dictionary built as a trie graph

	@author		Gordon Lee
*/

// Includes
#include "cDictionary.h"
#include <io.h>
#include <fstream>

// Namespace
namespace Test
{
////////////////////////////////////////////////////////////////////////////////
/**
	Constructor
*/
cDictionary::cDictionary()
{
}

/**
	Assignment Constructor

	@param char *_file
*/
cDictionary::cDictionary(char *_file)
{
	// Read from file
	std::ifstream _ifStream(_file, std::ios::in | std::ios::binary);
	char _buffer[BUF_LEN];

	if (!_ifStream.is_open())
		throw cDictionaryException();

	while (!_ifStream.eof())
	{
		_ifStream.getline(_buffer, BUF_LEN);
		m_trie.Insert(_buffer, 0, m_trie.Head());
	}

	_ifStream.close();
}

/**
	Deconstructor
*/
cDictionary::~cDictionary()
{
}

/**
	Trie

	@return cTrie *
*/
cTrie *cDictionary::Trie(void)
{
	return &m_trie;
}

/**
	Minimum Letters

	@return size_t
*/
size_t cDictionary::MinLetters(void)
{
	return m_minLetters;
}

/**
	Set Minimum Letters

	@param size_t _minLetters
*/
void cDictionary::SetMinLetters(size_t _minLetters)
{
	m_minLetters = _minLetters;
}
////////////////////////////////////////////////////////////////////////////////
}
