/*
	Engineering Test: Boggle Solver
	Gordon Lee

*******************************************************************************

	@filename	cList.cpp

	@description
		Class for a singly linked list of templatized nodes 

	@author		Gordon Lee
*/

// Includes
#include "cList.h"

// Namespace
namespace Test
{
////////////////////////////////////////////////////////////////////////////////
/**
	Constructor
*/
template <class T>
cList<T>::cList()
: m_head(NULL), m_tail(NULL), m_size(0)
{
}

/**
	Deconstructor
*/
template <class T>
cList<T>::~cList()
{
	while (m_head)
	{
		sNode *_temp = m_head;
		m_head = m_head->m_next;
		delete _temp;
	}
}

/**
	Empty

	@return bool
*/
template <class T>
bool cList<T>::Empty(void)
{
	if (m_size)
		return false;
	return true;
}

/**
	Find

	@param T _data

	@return bool
*/
template <class T>
bool cList<T>::Find(T _data)
{
	sNode *_temp = m_head;
	while (_temp)
	{
		if (_temp->m_data == _data)
			return true;
		_temp = _temp->m_next;
	}
	return false;
}

/**
	Front
	
	@return T
*/
template <class T>
T cList<T>::Front(void)
{
	return m_head->m_data;
}

/**
	Pop Front

	@return T
*/
template <class T>
T cList<T>::Pop_Front(void)
{
	if (Empty())
		return NULL;

	sNode *_temp = m_head;
	T _data = m_head->m_data;

	m_head = m_head->m_next;
	delete _temp;
	--m_size;

	return _data;
}

/**
	Push Back

	@param T _data
*/
template <class T>
void cList<T>::Push_Back(T _data)
{
	sNode *_node = new sNode;
	_node->m_data = _data;
	_node->m_next = NULL;
	if (m_tail != NULL)
		m_tail->m_next = _node;
	if (m_head == NULL)
		m_head = _node;
	m_tail = _node;
	++m_size;
}

/**
	Size

	@return size_t
*/
template <class T>
size_t cList<T>::Size(void)
{
	return m_size;
}
////////////////////////////////////////////////////////////////////////////////
}
