/*
	Engineering Test: Boggle Solver
	Gordon Lee

*******************************************************************************

	@filename	cMemoryPool.cpp

	@description
		Class for a memory pool class for managing different sizes of memory.

	@author		Gordon Lee
*/

// Includes
#include "cMemoryPool.h"

namespace Test
{
////////////////////////////////////////////////////////////////////////////////
/**
	Assignment Constructor

	@param unsigned _blockSize
	@param unsigned _blocksPerPage
*/
cMemoryPool::cMemoryPool(unsigned _blockSize, unsigned _blocksPerPage)
: m_blockSize(_blockSize), m_blocksPerPage(_blocksPerPage),
  m_pages(NULL), m_freeList(NULL),
  m_allocations(0), m_frees(0), m_numPages(0), m_numUsedBlocks(0), m_numFreeBlocks(0)
{
}

/**
	Deconstructor
*/
cMemoryPool::~cMemoryPool()
{
	Clear();
}

/**
	Allocate

	@return void *
*/
void *cMemoryPool::Allocate(void)
{
	if (m_numFreeBlocks == 0)
	{
		NewPage();
	}

	void *_block = reinterpret_cast<void *>(m_freeList);
	m_freeList = m_freeList->m_next;

	++m_allocations;
	++m_numUsedBlocks;
	--m_numFreeBlocks;

	return _block;
}

/**
	Free

	@param void *_object
*/
void cMemoryPool::Free(void *_object)
{
	sMemoryNode *_block = reinterpret_cast<sMemoryNode *>(_object);
	_block->m_next = m_freeList;
	m_freeList = _block;

	++m_frees;
	--m_numUsedBlocks;
	++m_numFreeBlocks;
}

/**
	Clear
*/
void cMemoryPool::Clear(void)
{
	sMemoryNode *_pageList = m_pages;

	// Loop through and free each page
	while (_pageList)
	{
		// Get page
		char *_page = reinterpret_cast<char *>(_pageList);

		// Go to next page
		_pageList = _pageList->m_next;

		delete [] _page;
	}
}

/**
	New Page
*/
void cMemoryPool::NewPage(void)
{
	// Calculate page size
	unsigned _size = unsigned(sizeof(sMemoryNode)) + (m_blocksPerPage * m_blockSize);

	// Allocate page
	sMemoryNode *_page = reinterpret_cast<sMemoryNode *>(new char[_size]);

	// Add page to list
	_page->m_next = m_pages;
	m_pages = _page;

	// Add blocks to list
	char *_block = reinterpret_cast<char *>(_page);
	_block += (unsigned)sizeof(sMemoryNode);
	for (unsigned i = 0; i < m_blocksPerPage; ++i)
	{
		// Add node
		sMemoryNode *_node = reinterpret_cast<sMemoryNode *>(_block);
		_node->m_next = m_freeList;
		m_freeList = _node;

		_block += m_blockSize;
	}

	m_numFreeBlocks += m_blocksPerPage;
	++m_numPages;
}
////////////////////////////////////////////////////////////////////////////////
}
