/*
	Engineering Test: Boggle Solver
	Gordon Lee

*******************************************************************************

	@filename	cMemoryPool.h

	@description
		Class for a memory pool class for managing different sizes of memory.

	@author		Gordon Lee
*/

#ifndef C_MEMORYPOOL_H
#define C_MEMORYPOOL_H
#pragma once

// Defines
#define NULL 0
#define DEFAULT_BLOCKSPERPAGE 100

namespace Test
{
////////////////////////////////////////////////////////////////////////////////
// Node struct
struct sMemoryNode
{
	sMemoryNode *m_next;
};

class cMemoryPool
{
public:
				cMemoryPool(unsigned _blockSize, unsigned _blocksPerPage = DEFAULT_BLOCKSPERPAGE);
				~cMemoryPool();

	void *		Allocate(void);
	void		Free(void *_object);
	void		Clear(void);

	unsigned	GetAllocations(void)			{	return m_allocations;	}
	unsigned	GetFrees(void)					{	return m_frees;			}
	unsigned	GetNumPages(void)				{	return m_numPages;		}
	unsigned	GetNumUsedBlocks(void)			{	return m_numUsedBlocks;	}
	unsigned	GetNumFreeBlocks(void)			{	return m_numFreeBlocks;	}
	unsigned	GetTotalMemoryAllocated(void)	{	return m_numPages * m_blocksPerPage * m_blockSize;	}

private:
	void		NewPage(void);

	// data members
	sMemoryNode *m_pages;
	sMemoryNode *m_freeList;

	unsigned m_blockSize;
	unsigned m_blocksPerPage;

	unsigned m_allocations;
	unsigned m_frees;
	unsigned m_numPages;
	unsigned m_numUsedBlocks;
	unsigned m_numFreeBlocks;
};
////////////////////////////////////////////////////////////////////////////////
}

#endif
